# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id: __init__.py 66 2008-03-07 06:36:15Z albrecht.andi $

"""Plugins

This package provides base classes for writing plugins. The `core`_ module
contains very basic classes, like `GenericPlugin`_ - the base class for all 
plugins. `mixins`_ has some mixin classes for common tasks.

Related documents
=================
 * `Plugin HowTo`_
 
.. _core: cf.plugins.core.html
.. _mixins: cf.plugins.mixins.html
.. _GenericPlugin: cf.plugins.core.GenericPlugin.html
.. _Plugin HowTo: http://code.google.com/p/crunchyfrog/wiki/PluginDevelopment
"""
